using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;

namespace FunTonn.MySqlDAL.SqlUtility {

    /// <summary>
    /// ֶαȽϱʽ
    /// </summary>
    public sealed class ColumnComparison {

        private SqlOperator _operator;
        private object[] _values;
        private SqlLogic _logic = SqlLogic.And;

        
        private ColumnComparison() { }// For test only

        public ColumnComparison(SqlOperator op, params char[] values) {
            try {
                SetValues(op, CopyFrom(values));
            } catch (Exception) {
                throw;
            }
        }

        public ColumnComparison(SqlOperator op, params string[] values) {
            try {
                SetValues(op, CopyFrom(values));
            } catch (Exception) {
                throw;
            }
        }

        public ColumnComparison(SqlOperator op, params int[] values) {
            try {
                SetValues(op, CopyFrom(values));
            } catch (Exception) {
                throw;
            }
        }

        public ColumnComparison(SqlOperator op, params decimal[] values) {
            try {
                SetValues(op, CopyFrom(values));
            } catch (Exception) {
                throw;
            }
        }

        public ColumnComparison(SqlOperator op, params DateTime[] values) {
            try {
                SetValues(op, CopyFrom(values));
            } catch (Exception) {
                throw;
            }
        }

        // publics
        /// <summary>
        /// ȡõǰֶζȽϱʽ֮߼
        /// </summary>
        public SqlLogic Logic {
            get { return _logic; }
            set { _logic = value; }
        }

        // privates
        private void SetValues(SqlOperator op, params object[] values) {
            _operator = op;
            switch (op) { 
                case SqlOperator.Equal:
                case SqlOperator.NotEqual:
                case SqlOperator.BigThan:
                case SqlOperator.LessThan:
                case SqlOperator.BigEqualThan:
                case SqlOperator.LessEqualThan:
                    CheckValuesNumber(values, 1); //  1 Ƚֵ
                break;

                case SqlOperator.Between:
                    CheckValuesNumber(values, 2); //  2 Ƚֵ
                break;

                default: 
                    CheckValuesNumber(values); 
                break;
            }
            _values = values;
        }

        private object[] CopyFrom(IEnumerable source) {
            IEnumerator ie = source.GetEnumerator();
            IList<object> result = new List<object>();
            while (ie.MoveNext()) {
                result.Add(ie.Current);
            }
            object[] target = new object[result.Count];
            result.CopyTo(target, 0);
            return target;
        }

        // 鴫ĲǷϹ
        // ϣ׳һ쳣
        private void CheckValuesNumber(object[] values) {
            if (values == null)
                throw new ArgumentNullException("values", "Ҫ 1 Ƚֵ");
            if (values.Length < 1)
                throw new ArgumentOutOfRangeException("values", "Ҫ 1 Ƚֵ");
        }

        // 鴫ĲǷϹ
        // ϣ׳һ쳣
        private void CheckValuesNumber(object[] values, int countAllowed) {
            if (values.Length != countAllowed)
                throw new SqlOperatorNotMatchArgumentException(_operator, countAllowed);
        }

        //private string FormatValue(string value) {
        //    return value.Replace(@"\",@"\\").Replace("'", @"\'");
        //}

        //private string FormatValue(string value, bool inLikeExpression) {
        //    if (!inLikeExpression)
        //        return FormatValue(value);
        //    return FormatValue(value).Replace("%",@"\%");
        //}

        // T-SQLȽַ
        // ڵʽͲʽ
        // ֻֻ1Ƚϲ
        private string BuildEqualString(string op) {
            return string.Format(
                " {0}{1}'{2}' ",
                "{0}", // һռλռλʾֶΣǰĵ
                op,
                TextFormat.ToEqualString(_values[0])
                );
        }

        // T-SQLȽַ
        //  between...and 
        // ֻֻ2Ƚϲ
        private string BuildBetweenString() {
            return string.Format(
                " ( {0} between '{1}' and '{2}' ) ",
                "{0}", // һռλռλʾֶΣǰĵ
                TextFormat.ToEqualString(_values[0]),
                TextFormat.ToEqualString(_values[1])
                );
        }

        // T-SQLȽַ
        //  in  not in 
        // ֻھ1ϵıȽϲ
        private string BuildInString(string op) {
            return string.Format(
                " {0} {1} ({2}) ",
                "{0}", // һռλռλʾֶΣǰĵ
                op,
                GetValueString()
                );
        }

        private string BuildLikeString(string op, SqlOperator likeType) { 
            StringBuilder sb = new StringBuilder();
            bool writeFlag = false;
                sb.Append(" (");
                for (int i = 0; i < _values.Length; i++) {
                    if (writeFlag)
                        sb.Append(GetLogicString());

                    sb.AppendFormat(
                        "{0} {1} {2}",
                        "{0}",
                        op,
                        GetLikeOp(likeType, _values[i].ToString())
                        );
                    writeFlag = true;
                }
                sb.Append(") ");

                return sb.ToString();
        }

        private string GetValueString() {
            StringBuilder sb = new StringBuilder();
            bool flag = false;
            foreach (object o in _values) {
                if (flag)
                    sb.Append(',');

                if (o == null)
                    continue;
                //if (o.ToString() == string.Empty)
                //    continue;

                sb.AppendFormat(
                    "'{0}'", TextFormat.ToEqualString(o)
                    );
                flag = true;
            }
            return sb.ToString();
        }

        private string GetLikeOp(SqlOperator opType, string value) {
            switch (opType) {
                case SqlOperator.StartWith: return string.Format("'{0}%'", TextFormat.ToEqualString(value));
                case SqlOperator.EndWidth: return string.Format("'%{0}'", TextFormat.ToEqualString(value));
                case SqlOperator.Like: return string.Format("'%{0}%'", TextFormat.ToEqualString(value));
                default: throw new ApplicationException("δע opType.");
            }
        }

        public string GetLogicString() {
            if (_logic == SqlLogic.Or)
                return " or ";
            return " and ";
        }

        // overrides
        public override string ToString() {
            switch (_operator) {
                case SqlOperator.Equal: return BuildEqualString("=");
                case SqlOperator.NotEqual: return BuildEqualString("<>");
                case SqlOperator.In: return BuildInString("in");
                case SqlOperator.NotIn: return BuildInString("NOT IN");
                case SqlOperator.Between: return BuildBetweenString();
                case SqlOperator.BigThan: return BuildEqualString(">");
                case SqlOperator.LessThan: return BuildEqualString("<");
                case SqlOperator.BigEqualThan: return BuildEqualString(">=");
                case SqlOperator.LessEqualThan: return BuildEqualString("<=");
                case SqlOperator.NotLike: return BuildLikeString("not like", SqlOperator.Like);
                default: return BuildLikeString("like", _operator);
            }
        }

        #region Test methods (debug model only)
#if DEBUG
        private void Test_CopyFrom() {
            string[] source = new string[] { "e", "1", "f", "t", "eee" };
            object[] oa = CopyFrom(source);
            Console.WriteLine("Item Count: {0}", oa.Length);
            foreach (object o in oa) {
                Console.WriteLine(o);
            }
        }
        private void Test_ToString() {
            string[] values = new string[]{"wfyfng'u","d","jinxiang"};
            ColumnComparison cc = new ColumnComparison(SqlOperator.Like, values);
            cc.Logic = SqlLogic.And;
            Console.WriteLine(cc.ToString());
            cc.Logic = SqlLogic.Or;
            Console.WriteLine(cc.ToString());
            cc = new ColumnComparison( SqlOperator.In, values);
            Console.WriteLine(cc.ToString());
        }
        //private void Test_FormatValue() {
        //    string text = @"*'\%%\";
        //    Console.WriteLine(FormatValue(text, false));
        //    Console.WriteLine(FormatValue(text, true));
        //}
#endif
        #endregion

    }
}
