using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Text;
using MySql.Data.MySqlClient;

namespace FunTonn.MySqlDAL.SqlUtility {

    public abstract class TableAccess<T> : ReadonlyTableAccess<T> {

        /// <summary>
        /// ʹָʵSQLеĲֵڱ޸ĵȶ
        /// </summary>
        /// <param name="entity">Ѿʵ</param>
        /// <returns></returns>
        abstract internal ColumnParameter[] FillColumnParameters(T entity);

        /// <summary>
        /// SQL Insert
        /// </summary>
        /// <param name="entity"></param>
        /// <param name="parameters"></param>
        /// <returns></returns>
        virtual protected string BuildInsertSql(T entity, out MySqlParameter[] parameters) {
            parameters = null;
            ColumnParameter[] cps = FillColumnParameters(entity);
            CheckColumnParameter(cps);

            StringBuilder sqlFields = new StringBuilder();
            StringBuilder sqlValues = new StringBuilder();
            ColumnParameter tempParameter = null;
            IList<MySqlParameter> sortedParams = new List<MySqlParameter>(cps.Length);
            foreach (TableColumn column in TableColumns) {
                if (column.IsReadonly)
                    continue;

                tempParameter = GetParameterByColumn(column, cps);
                if (tempParameter == null)
                    throw new ColumnParameterNotSuppliedException(column.Name);

                sqlFields.AppendFormat("{0},", tempParameter.Column.Name);
                sqlValues.AppendFormat("{0},", tempParameter.Parameter.ParameterName);
                System.Diagnostics.Debug.WriteLine(tempParameter.Parameter.ParameterName);
                sortedParams.Add(tempParameter.Parameter);
            }
            parameters = new MySqlParameter[sortedParams.Count];
            sortedParams.CopyTo(parameters, 0);
            // Insert
            sqlFields.Remove(sqlFields.Length - 1, 1); // ɾһ
            sqlValues.Remove(sqlValues.Length - 1, 1); // ɾһ

            return string.Format(
                "insert into {0} ({1}) values ({2})",
                TableName,
                sqlFields.ToString(),
                sqlValues.ToString()
                );
        }

        /// <summary>
        ///  SQL Update ޸ģ
        /// </summary>
        /// <param name="primaryKeyValue">ֵ</param>
        /// <param name="newEntity"></param>
        /// <param name="bound"></param>
        /// <param name="excluded"></param>
        /// <param name="parameters"></param>
        /// <returns></returns>
        virtual internal string BuildUpdateSql(object primaryKeyValue, T newEntity, ModifyBounds bound, TableColumn[] excluded, out MySqlParameter[] parameters) {
            parameters = null;
            ColumnParameter[] cps = FillColumnParameters(newEntity);
            CheckColumnParameter(cps);
            ColumnParameter tempParameter = null;
            IList<MySqlParameter> sortedParams = new List<MySqlParameter>(cps.Length);

            // ѡҪ޸ĵֶ
            // עųreadonlyֶ
            TableColumnCollection validColumns = GetColumnToUpdate(bound, excluded);

            // ʼSQL
            StringBuilder sqlUpdate = new StringBuilder();
            sqlUpdate.AppendFormat("update {0} set ", TableName);
            bool columnAdded = false;
            foreach (TableColumn column in validColumns) {
                if (column.IsReadonly)
                    continue;

                tempParameter = GetParameterByColumn(column, cps);
                if (tempParameter == null)
                    throw new ColumnParameterNotSuppliedException(column.Name);

                sqlUpdate.AppendFormat("{0}={1},", tempParameter.Column.Name, tempParameter.Parameter.ParameterName);
                sortedParams.Add(tempParameter.Parameter);
                if (!columnAdded) columnAdded = true;
            }
            if (columnAdded) {
                sqlUpdate.Remove(sqlUpdate.Length - 1, 1); // ɾһ
            }
            sqlUpdate.AppendFormat(" where {0}=?primaryKeyValue", TablePrimaryColumn.Name);
            sortedParams.Add(new MySqlParameter("?primaryKeyValue", primaryKeyValue));

            parameters = new MySqlParameter[sortedParams.Count];
            sortedParams.CopyTo(parameters, 0);

            return sqlUpdate.ToString();
        }

        /// <summary>
        /// ΪĿݱԾӵֶοٸֵ
        /// ִֵSave()Modify()Ȳʱʹá
        /// ʹø÷ΪÿһѾӵֶθֵΪReadonlyֶ
        /// </summary>
        /// <param name="values"></param>
        /// <returns></returns>
        internal ColumnParameter[] BuildColumnParameter(params object[] values) {
            if (values == null)
                throw new ArgumentNullException("values");
            // 鴫ֵǷݱֶһ
            int targetCount = TableColumns.Count;
            if (values.Length != targetCount)
                throw new ArgumentOutOfRangeException("values", "values Ԫظݱֶһ¡");

            ColumnParameter[] result = new ColumnParameter[targetCount];
            for (int i = 0; i < targetCount; i++) {
                result[i] = new ColumnParameter(TableColumns[i], CreateMySqlParameter(i, values[i]));
            }
            return result;
        }

        /// <summary>
        /// һʵ嵽ݿ
        /// </summary>
        /// <param name="entity"></param>
        /// <param name="tran"></param>
        virtual internal void Save(T entity, MySqlTransaction tran) {
            try {
                MySqlParameter[] parameters;
                string sql = BuildInsertSql(entity, out parameters);
                if (tran == null)
                    DBUtility.ExecuteNonQuery(CommandType.Text, sql, parameters);
                else
                    DBUtility.ExecuteNonQuery(tran, CommandType.Text, sql, parameters);
            } catch (Exception) {
                throw;
            }
        }

        /// <summary>
        /// ޸ľֵָС
        /// ĿݱûУ׳һ쳣
        /// </summary>
        /// <param name="primaryKeyValue">Ҫ޸ĵĿֵ</param>
        /// <param name="newEntity">µʵ</param>
        /// /// <param name="tran"></param>
        /// <param name="bound">Ҫ޸ĵеķΧ޸л޸κУ</param>
        /// <param name="excluded">޸ķΧӰ</param>
        /// <returns></returns>
        /// <exception cref="System.NotSupportedException">TablePrimaryColumnΪnullʱ쳣</exception>
        internal int Modify(object primaryKeyValue, T newEntity, MySqlTransaction tran, ModifyBounds bound, params TableColumn[] excluded) {
            if (TablePrimaryColumn == null)
                throw new NotSupportedException("This operation was NOT supported when TablePrimaryColumn equals null.");
            try {
                MySqlParameter[] parameters;
                string sql = BuildUpdateSql(primaryKeyValue, newEntity, bound, excluded, out parameters);
                System.Diagnostics.Trace.WriteLine(sql);
                if (tran == null)
                    return DBUtility.ExecuteNonQuery(CommandType.Text, sql, parameters);
                else
                    return DBUtility.ExecuteNonQuery(tran, CommandType.Text, sql, parameters);
            } catch (Exception) {
                throw;
            }
        }

        /// <summary>
        /// ɾļ¼
        /// </summary>
        /// <param name="filter"></param>
        /// <param name="tran"></param>
        /// <returns></returns>
        internal int Delete(SearchConditionCollection filter, MySqlTransaction tran) {
            try {
                // ûָκ
                if (filter.Count < 1)
                    return 0;

                string sql = string.Format(
                    "delete from {0} where {1}",
                    TableName, filter.ToString()
                    );
                if (tran == null)
                    return DBUtility.ExecuteNonQuery(CommandType.Text, sql, null);
                else
                    return DBUtility.ExecuteNonQuery(tran, CommandType.Text, sql, null);
            } catch (Exception) {
                throw;
            }
        }

        /// <summary>
        /// һʵ嵽ݿ
        /// </summary>
        /// <param name="entity"></param>
        public void Save(T entity) {
            try {
                Save(entity, null);
            } catch (Exception) {
                throw;
            }
        }

        /// <summary>
        /// ʹָʵ޸ľֵָ
        /// </summary>
        /// <param name="primaryKeyValue">Ҫ޸ĵĿֵ</param>
        /// <param name="newEntity">µʵ</param>
        /// <returns></returns>
        public int Modify(object primaryKeyValue, T newEntity) {
            return Modify(primaryKeyValue, newEntity, null, ModifyBounds.All, null);
        }

        /// <summary>
        /// ɾļ¼
        /// </summary>
        /// <param name="filter"></param>
        /// <returns></returns>
        public int Delete(SearchConditionCollection filter) {
            return Delete(filter, null);
        }

        // Privates
        private void CheckColumnParameter(ColumnParameter[] waitToCheck) {
            if (waitToCheck == null || waitToCheck.Length<1)
                throw new ArgumentNullException(
                    string.Format("Method FillColumnParameters({0}) returns value can NOT be null or empty.", typeof(T))
                    );
        }

        private ColumnParameter GetParameterByColumn(TableColumn column, ColumnParameter[] source) {
            foreach (ColumnParameter cp in source) {
                if (cp.Column.Equals(column))
                    return cp;
            }
            return null;
        }

        private MySqlParameter CreateMySqlParameter(int columnIndex, object value) {
            MySqlParameter param = new MySqlParameter();
            param.ParameterName = "?" + TableName + "_" + columnIndex.ToString();
            if (value == null)
                param.Value = DBNull.Value;
            else
                param.Value = value;
            return param;
        }

        // ȡҪִи²ݱֶ
        // صбвֶֻ
        private TableColumnCollection GetColumnToUpdate(ModifyBounds bound, params TableColumn[] excluded) {
            TableColumnCollection validColumns = new TableColumnCollection();
            if(excluded == null) excluded = new TableColumn[0];
            // ĬΪеֶζ£ôųֶξЧֶ
            // 򣬹
            if (bound == ModifyBounds.None) {
                validColumns.AddRange(excluded);
            } else {
                TableColumnCollection allColumns = TableColumns; // ֶ
                foreach (TableColumn tc in allColumns) {
                    if (!((IList)excluded).Contains(tc)) {
                        validColumns.Add(tc);
                    }
                }
            }
            // Чֶбɾ
            validColumns.Remove(TablePrimaryColumn);
            return validColumns;
        }


    }
}
