using System;
using System.Collections.Generic;

namespace FunTonn.MySqlDAL.SqlUtility {

    /// <summary>
    /// ݱֶμ
    /// ܴӸ
    /// </summary>
    public sealed class TableColumnCollection : IList<TableColumn> {

        IList<TableColumn> list = new List<TableColumn>();

        public TableColumnCollection() {
            list = new List<TableColumn>();
        }

        public TableColumnCollection(int capacity) {
            list = new List<TableColumn>(capacity);
        }

        #region IList<TableColumn> Members

        public int IndexOf(TableColumn item) {
            return list.IndexOf(item);
        }

        public void Insert(int index, TableColumn item) {
            list.Insert(index, item);
        }

        public void RemoveAt(int index) {
            list.RemoveAt(index);
        }

        public TableColumn this[int index] {
            get {
                return list[index];
            }
            set {
                list[index] = value;
            }
        }

        public TableColumn this[string name] {
            get {
                foreach (TableColumn tc in list) {
                    if (string.Compare(tc.Name, name, true) == 0)
                        return tc;
                }
                return null;
            }
        }

        #endregion

        #region ICollection<TableColumn> Members

        public void Add(TableColumn item) {
            if (item == null)
                throw new ArgumentNullException("item");
            if (list.Contains(item))
                throw new Exception(string.Format("Column {0} already added.", item.Name));
            list.Add(item);
        }

        public void AddRange(TableColumn[] items) {
            if (items == null)
                throw new ArgumentNullException("items");
            foreach (TableColumn tc in items) {
                Add(tc);
            }
        }

        public void AddRange(TableColumnCollection list) {
            if (list == null)
                throw new ArgumentNullException("list");
            foreach (TableColumn tc in list) {
                Add(tc);
            }
        }

        public void Clear() {
            list.Clear();
        }

        public bool Contains(TableColumn item) {
            return list.Contains(item);
        }

        public void CopyTo(TableColumn[] array, int arrayIndex) {
            list.CopyTo(array, arrayIndex);
        }

        public int Count {
            get {
                return list.Count;
            }
        }

        public bool IsReadOnly {
            get { return list.IsReadOnly; }
        }

        public bool Remove(TableColumn item) {
            return list.Remove(item);
        }

        #endregion

        #region IEnumerable<TableColumn> Members

        public IEnumerator<TableColumn> GetEnumerator() {
            return list.GetEnumerator();
        }

        #endregion

        #region IEnumerable Members

        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator() {
            return list.GetEnumerator();
        }

        #endregion


    }
}
